@@ -1,9 +1,7 @@
-from __future__ import division, absolute_import, print_function
-
 import unittest
 import numpy.testing as testing
 import numpy as np
-import healpy as hp
+import hpgeom as hpg
 from numpy import random
 
 import healsparse
@@ -20,7 +18,7 @@ class ValidAreaTestCase(unittest.TestCase):
         nside_map = 64
 
         n_rand = 1000
-        r_indices = np.random.choice(np.arange(hp.nside2npix(nside_map)),
+        r_indices = np.random.choice(np.arange(hpg.nside_to_npixel(nside_map)),
                                      size=n_rand, replace=False)
 
         for dt in [np.float64, np.int64]:
@@ -32,6 +30,10 @@ class ValidAreaTestCase(unittest.TestCase):
 
             # Fill up the maps
             sparse_map.update_values_pix(r_indices, np.ones(n_rand, dtype=dt))
-            testing.assert_equal(sparse_map.get_valid_area(), n_rand*hp.nside2pixarea(nside_map,
-                                 degrees=True))
+            testing.assert_equal(sparse_map.get_valid_area(),
+                                 n_rand*hpg.nside_to_pixel_area(nside_map, degrees=True))
             testing.assert_equal(sparse_map.n_valid, n_rand)
+
+
+if __name__ == '__main__':
+    unittest.main()
